@@ -2,8 +2,8 @@
                           LOW_netSegment.h  -  description
                              -------------------
     begin                : Sun Jul 7 2002
-    copyright            : (C) 2002 by Harald Roelle, Helmut Reiser
-    email                : roelle@informatik.uni-muenchen.de, reiser@informatik.uni-muenchen.de
+    copyright            : (C) 2002 by Harald Roelle
+    email                : roelle@informatik.uni-muenchen.de
  ***************************************************************************/
 
 /***************************************************************************
@@ -22,8 +22,10 @@
 #include <algorithm>
 
 #include "LOW_types.h"
+#include "LOW_objectIDFactory.h"
 #include "LOW_device.h"
 #include "LOW_link.h"
+#include "LOW_objectSynchronizer.h"
   
 
 class LOW_device; // Forward declaration to avoid loops.
@@ -36,15 +38,21 @@ class LOW_device; // Forward declaration to avoid loops.
 
     Objects for devices are automatically instanciated via LOW_deviceFactory.
     The objects are cached in two maps:
+    
       - aliveDevMap: Contains currently present devices on the segment.
+
       - graveyardMap: Contains formerly present devices on the segment.
+
     This means, that once a device is not visible any more, the device object
     is not deleted but burried on the graveyard. Once the device is visible again,
     it is revitalized.
 
-    @author Harald Roelle, Helmut Reiser
+    This class is thread-safe.
+    
+    @author Harald Roelle
+    @author Parts of the documentation by Dallas Semiconductors / Maxim Integrated Products
   */
-class LOW_netSegment
+class LOW_netSegment : public LOW_objectSynchronizer
 {
 
 //=======================================================================================
@@ -67,7 +75,7 @@ public:
   // type definitions
   //
       
-  typedef uint32_t segmentID_t;  /**< segment ID type */
+  typedef LOW_objectIDFactory::objectID_t segmentID_t;  /**< segment ID type */
   
   typedef std::vector<LOW_netSegment*> netSegPtrVec_t;  /**< Vector type of class pointers. */
   
@@ -80,13 +88,13 @@ public:
   /** Get the LOW_link instance associated with this segment.
       @return  Reference to the link instance.
    */
-  LOW_link& getLink();
+  virtual LOW_link& getLink();
   
   
   /** Get wether there is an external power line on the segment.
       @returns  Wether there is an external power line on the segment.
    */
-  bool getHasExternalPower() const;
+  virtual bool getHasExternalPower() const;
 
   
   /** Get a specific device.
@@ -115,7 +123,7 @@ public:
       
       @return Vector of found devices.
    */
-  template<class devType> vector<devType*> getDevices() const;
+  template<class devType> std::vector<devType*> getDevices() const;
 
     
   /** Search for devices on the segment.
@@ -132,7 +140,7 @@ public:
                            requested type.   
       @return Vector of found devices.
    */
-  template<class devType> vector<devType*> searchDevices( const bool inOnlyAlarm = false);
+  template<class devType> std::vector<devType*> searchDevices( const bool inOnlyAlarm = false);
 
     
   /** Verify existance of a specific device on the segment.
@@ -149,8 +157,8 @@ public:
       @param  inDoReset    Whether to reset the bus afterwards.   
       @return Boolean indicates wheter the device could be found or not.
    */
-  bool verifyDevice( const LOW_deviceID inDevID, const bool inOnlyAlarm = false, 
-                     const bool inDoReset = true);
+  virtual bool verifyDevice( const LOW_deviceID inDevID, const bool inOnlyAlarm = false, 
+                             const bool inDoReset = true);
   
 
             
@@ -165,6 +173,16 @@ protected:
   friend class LOW_network;  /**< Enable access to static pseudo constructor and destructor */
   friend class LOW_device;   /**< Enable access to unregister a device on destruction. */
   
+
+  
+  //=====================================================================================
+  //
+  // static attributes
+  //
+
+  __LOW_SYNCHRONIZE_DEFINE_PROTECTED_LOCK__
+
+
   
   //=====================================================================================
   //
@@ -174,7 +192,8 @@ protected:
   /** Destructor.
       Destroys all objects of devices (in both maps).
    */
-  ~LOW_netSegment();
+  virtual ~LOW_netSegment();
+
 
   
   //=====================================================================================
@@ -188,6 +207,7 @@ protected:
       @param  inLink  The link to be searched for network segments.
    */
   static netSegPtrVec_t newSegmentsFromLink( LOW_link &inLink);
+
   
     
   //=====================================================================================
@@ -200,13 +220,13 @@ protected:
   
       @param inDev  Device to unregister.
    */
-  void unregisterDevice( const LOW_device *inDev);
+  virtual void unregisterDevice( const LOW_device *inDev);
   
   
   
   //=====================================================================================
   //
-  // addressing/searching methods
+  // protected addressing/searching methods
   //
   
   /** Select a single device on a link.
@@ -221,7 +241,7 @@ protected:
       @param inDevice  Device to select.
       @throw noDevice_error  Thrown when no device was found on the bus.
    */
-  void cmd_MatchROM( const LOW_device *inDevice) const;
+  virtual void cmd_MatchROM( const LOW_device *inDevice) const;
 
     
   /** Select all devices on a link.
@@ -234,7 +254,7 @@ protected:
 
       @throw noDevice_error  Thrown when no device was found on the bus.
    */
-  void cmd_SkipROM() const;
+  virtual void cmd_SkipROM() const;
   
   
   /** Read ID number from bus.
@@ -250,7 +270,7 @@ protected:
       @return  ID of the found device.
       @throw noDevice_error  Thrown when no device was found on the bus.
    */
-  LOW_deviceID cmd_ReadROM() const;
+  virtual LOW_deviceID cmd_ReadROM() const;
 
   
   /** Scan the bus for devices.
@@ -272,8 +292,8 @@ protected:
       @return  List of found IDs.
       @throw noDevice_error  Thrown when no device was found on the bus.
    */  
-  LOW_deviceID::deviceIDVec_t cmd_SearchROM( const bool inOnlyAlarm = false,
-                                             const LOW_deviceIDRaw::devFamCode_t inFamCode = LOW_device::anyDev_famCode) const;
+  virtual LOW_deviceID::deviceIDVec_t cmd_SearchROM( const bool inOnlyAlarm = false,
+                                                     const LOW_deviceIDRaw::devFamCode_t inFamCode = LOW_device::anyDev_famCode) const;
   
   
   /** Scan the bus for a specific device.
@@ -284,28 +304,19 @@ protected:
       @return   Indicated whether the device was found or not.
       @throw noDevice_error  Thrown when no device was found on the bus.
    */  
-  bool cmd_SearchROMVerify( const LOW_deviceID inDevID, const bool inOnlyAlarm = false, 
-                            const bool inDoReset = true) const;
+  virtual bool cmd_SearchROMVerify( const LOW_deviceID inDevID, const bool inOnlyAlarm = false,
+                                    const bool inDoReset = true) const;
 
   
 
 //=======================================================================================
 private:
   
-  //=====================================================================================
-  //
-  // static attributes
-  //
-  
-  static segmentID_t  segmentCount; /**< Incremented on instance creation to get individual IDs. */
-  
-  
   //=====================================================================================
   //
   // attributes
   //
   
-  
   const segmentID_t         segmentID;         /**< Identifier of the segment. */
   LOW_link                  &link;             /**< Link this segment is present on */
   bool                      hasExternalPower;  /**< Whether there is an additional line for external power supply available */
@@ -330,7 +341,7 @@ private:
   // operator overloading
   //
 
-  bool operator==(LOW_netSegment &inSegment) const;  /**< Comparison based on segmentID. */
+  virtual bool operator==(LOW_netSegment &inSegment) const;  /**< Comparison based on segmentID. */
 
 
   //=====================================================================================
@@ -353,7 +364,7 @@ private:
       
       @param  inDev  The device to move.
    */
-  void buryDevice( const LOW_device *inDev);
+  virtual void buryDevice( const LOW_device *inDev);
   
   
   /** Move a device from the graveyard to the alive map.
@@ -361,7 +372,7 @@ private:
       
       @param  inDev  The device to move.
    */
-  void revitalizeDevice( const LOW_device *inDev);
+  virtual void revitalizeDevice( const LOW_device *inDev);
   
   
   /** Add a device to the alive map.
@@ -373,7 +384,7 @@ private:
       @param  inDevID  The ID of the device to add.
       @return The added device.
    */
-  LOW_device* addDevice( const LOW_deviceID inDevID);
+  virtual LOW_device* addDevice( const LOW_deviceID inDevID);
   
   
   /** Removes a device from either the alive map or the graveyard.
@@ -381,7 +392,7 @@ private:
   
       @param  inDev  The device to remove.
    */
-  void removeDevice( const LOW_device *inDev);
+  virtual void removeDevice( const LOW_device *inDev);
 
 };
 
@@ -399,6 +410,7 @@ private:
 
 template<class devType> devType* LOW_netSegment::getDevice( const LOW_deviceID inDevID)
 {
+  __LOW_SYNCHRONIZE_METHOD_READ__
 
   if ( devType::familyCode!=LOW_device::anyDev_famCode && inDevID.getFamilyCode()!=devType::familyCode)
     throw LOW_device::familyMismatch_error( "Requested type does not match given device ID", __FILE__, __LINE__);
@@ -412,9 +424,11 @@ template<class devType> devType* LOW_netSegment::getDevice( const LOW_deviceID i
 }
 
 
-template<class devType> vector<devType*> LOW_netSegment::getDevices() const
+template<class devType> std::vector<devType*> LOW_netSegment::getDevices() const
 {
-  vector<devType*> retVal;
+  __LOW_SYNCHRONIZE_METHOD_READ_WEAK__
+
+  std::vector<devType*> retVal;
 
   // look in the map of alive devices
   for( LOW_device::deviceMap_t::const_iterator a=aliveDevMap.begin(); a!=aliveDevMap.end(); ++a) {
@@ -426,15 +440,17 @@ template<class devType> vector<devType*> LOW_netSegment::getDevices() const
 }
   
   
-template<class devType> vector<devType*> LOW_netSegment::searchDevices( const bool inOnlyAlarm)
+template<class devType> std::vector<devType*> LOW_netSegment::searchDevices( const bool inOnlyAlarm)
 {
+  __LOW_SYNCHRONIZE_METHOD_WRITE_WEAK__
+  
   LOW_deviceID::deviceIDVec_t idList = cmd_SearchROM( inOnlyAlarm, devType::familyCode);
-  vector<devType*>            retVal = vector<devType*>( idList.size());
+  std::vector<devType*>       retVal = std::vector<devType*>( idList.size());
 
   // in case of NOT alarm search, remember the active devices
   LOW_device::devPtrVec_t   formerlyActiveDevs;
   if ( !inOnlyAlarm ) {
-    vector<devType*> tmpVec = getDevices<devType>();
+    std::vector<devType*> tmpVec = getDevices<devType>();
     for( unsigned int a=0; a<tmpVec.size(); a++)
       formerlyActiveDevs.push_back( tmpVec[a]);
   }
@@ -463,6 +479,8 @@ template<class devType> vector<devType*> LOW_netSegment::searchDevices( const bo
 
 template<class castType> inline castType* LOW_netSegment::devicePtr_cast( LOW_device *inPtr) const
 {
+  // no locking necessary
+  
   castType* retVal = dynamic_cast<castType*>(inPtr);
   
   if ( retVal == 0 )
