@@ -2,8 +2,8 @@
                           LOW_devDS2406.h  -  description
                              -------------------
     begin                : Fri Aug 23 2002
-    copyright            : (C) 2002 by Harald Roelle, Helmut Reiser
-    email                : roelle@informatik.uni-muenchen.de, reiser@informatik.uni-muenchen.de
+    copyright            : (C) 2002 by Harald Roelle
+    email                : roelle@informatik.uni-muenchen.de
  ***************************************************************************/
 
 /***************************************************************************
@@ -28,30 +28,33 @@
 
     DS2406 features:
 
-   - Open drain PIO pins are controlled and their logic level can be determined
-     over 1-Wire bus for closed-loop control
-   - Replaces and is fully compatible with DS2407 but no user-programmable
-     power-on settings and no Hidden Mode
-   - PIO channel A sink capability of 50mA at 0.4V with soft turn-on; channel B 8mA at 0.4V
-   - Maximum operating voltage of 13V at PIO-A, 6.5V at PIO-B
-   - 1024 bits user-programmable OTP EPROM
-   - User-programmable status memory to control the device
-   - Multiple DS2406 can be identified on a common 1-Wire bus and be turned on or off
-     independently of other devices on the bus
-   - Unique, factory-lasered and tested 64-bit registration number (8-bit family code + 48-bit
-     serial number + 8-bit CRC tester) assures error-free selection and absolute identity
-     because no two parts are alike
-   - On-chip CRC16 generator allows detection of data transfer errors
-   - Built-in multidrop controller ensures compatibility with other 1-Wire net products
-   - Reduces control, address, data, programming and power to a single data pin
-   - Directly connects to a single port pin of a microprocessor and communicates
-     at up to 16.3 kbits/s
-   - Supports Conditional Search with userselectable condition
-   - Vcc bondout for optional external supply to the device (TSOC package only)
-   - 1-Wire communication operates over a wide voltage range of 2.8V to 6.0V from -40C to +85C
-   - Low cost TO-92 or 6-pin TSOC surface mount package
+    - Open drain PIO pins are controlled and their logic level can be determined
+      over 1-Wire bus for closed-loop control
+    - Replaces and is fully compatible with DS2407 but no user-programmable
+      power-on settings and no Hidden Mode
+    - PIO channel A sink capability of 50mA at 0.4V with soft turn-on; channel B 8mA at 0.4V
+    - Maximum operating voltage of 13V at PIO-A, 6.5V at PIO-B
+    - 1024 bits user-programmable OTP EPROM
+    - User-programmable status memory to control the device
+    - Multiple DS2406 can be identified on a common 1-Wire bus and be turned on or off
+      independently of other devices on the bus
+    - Unique, factory-lasered and tested 64-bit registration number (8-bit family code + 48-bit
+      serial number + 8-bit CRC tester) assures error-free selection and absolute identity
+      because no two parts are alike
+    - On-chip CRC16 generator allows detection of data transfer errors
+    - Built-in multidrop controller ensures compatibility with other 1-Wire net products
+    - Reduces control, address, data, programming and power to a single data pin
+    - Directly connects to a single port pin of a microprocessor and communicates
+      at up to 16.3 kbits/s
+    - Supports Conditional Search with userselectable condition
+    - Vcc bondout for optional external supply to the device (TSOC package only)
+    - 1-Wire communication operates over a wide voltage range of 2.8V to 6.0V from -40C to +85C
+    - Low cost TO-92 or 6-pin TSOC surface mount package
 
-   @author Harald Roelle, Helmut Reiser
+    This class is thread-safe.
+    
+    @author Harald Roelle
+    @author  Parts of the documentation by Dallas Semiconductors / Maxim Integrated Products
  */
 class LOW_devDS2406 : public LOW_device  {
 
@@ -77,7 +80,7 @@ public:
   static const LOW_deviceIDRaw::devFamCode_t  familyCode = 0x12;
 
   /** Family name of this specific device. */
-  static const string familyName;
+  static const std::string familyName;
   
   static const owCommand_t  ReadMemory_COMMAND         = 0xf0; /**< 1-Wire command byte constant */
   static const owCommand_t  ExtendedReadMemory_COMMAND = 0xa5; /**< 1-Wire command byte constant */
@@ -199,7 +202,7 @@ public:
     /** Destructor.
         Releases the lock on the device.
      */
-    ~cmd_ChannelAccess();
+    virtual ~cmd_ChannelAccess();
     
     //-------------------------------------------------------------------------------------
     // methods
@@ -207,38 +210,38 @@ public:
 
     /** Get the channel info read after sending command.
      */
-    channelInfo_t& getChannelInfo();
+    virtual channelInfo_t& getChannelInfo();
 
     /** Receive 1 bit from the device.
         @return  Bit that was reveived.
      */
-    bool readDataBit() const;
+    virtual bool readDataBit() const;
 
     /** Receive 1 byte from the device.
         @return  Byte that was reveived.
      */
-    uint8_t readDataByte() const;
+    virtual uint8_t readDataByte() const;
 
     /** Receive a block of bytes from the device.
-      @param  outBytes   Values that were reveived. Read length is determined
-                         by the preset length of the vector.
+        @param  outBytes   Values that were reveived. Read length is determined
+                           by the preset length of the vector.
      */
-    void readData( byteVec_t &outBytes) const;
+    virtual void readData( byteVec_t &outBytes) const;
 
     /** Send 1 bit to the device.
         @param   inSendBit   Bit to send.
      */
-    void writeData( const bool inSendBit) const;
+    virtual void writeData( const bool inSendBit) const;
 
     /** Send 1 byte to the device.
         @param   inSendByte  Byte to send.
      */
-    void writeData( const uint8_t inSendByte) const;
+    virtual void writeData( const uint8_t inSendByte) const;
 
     /** Send block of bytes to the device.
         @param  inSendBytes  Block of bytes to send.
      */
-    void writeData( const byteVec_t &inSendBytes) const;
+    virtual void writeData( const byteVec_t &inSendBytes) const;
   
   //-------------------------------------------------------------------------------------
   private:
@@ -261,7 +264,7 @@ public:
 
   /** Destructor.
    */
-  ~LOW_devDS2406();
+  virtual ~LOW_devDS2406();
 
   
   //=====================================================================================
@@ -272,19 +275,19 @@ public:
   /** Get the device's family name.
       @return Family name of the device.
    */
-  virtual string getFamilyName() const { return familyName; };
+  virtual std::string getFamilyName() const { return familyName; };
 
   
   /** Get wether the device is externally powered.
       @return  Boolean indicates external power.
    */
-  bool getIsExternalPowered() const;
+  virtual bool getIsExternalPowered() const;
 
   
   /** Get wether the second PIO channel is present.
       @return  Boolean indicates presence of channel B.
    */
-  bool getHasPioB() const;
+  virtual bool getHasPioB() const;
 
   
   /** Get the current search condition.
@@ -293,7 +296,7 @@ public:
       @param  outStatusRegister  Pointer to status register struture. Results
                                  are filled in there.
    */
-  void getSearchCondition( LOW_devDS2406::statusRegister_t *outStatusRegister) const;
+  virtual void getSearchCondition( LOW_devDS2406::statusRegister_t *outStatusRegister) const;
 
     
   /** Set the search condition and status of the PIO transistors.
@@ -371,9 +374,9 @@ public:
 
       @throw devDS2406_error  Thrown when channel B is selected without being present.
    */
-  void setSearchCondition( const chanSelect_t inChanSelect, const sourceSelect_t inSourceSelect, 
-                           const activePolarity_t inPolaritySelect,
-                           const pioTransistor_t inPioATrans, const pioTransistor_t inPioBTrans) const;
+  virtual void setSearchCondition( const chanSelect_t inChanSelect, const sourceSelect_t inSourceSelect,
+                                   const activePolarity_t inPolaritySelect,
+                                   const pioTransistor_t inPioATrans, const pioTransistor_t inPioBTrans) const;
 
 
   /** Read from EPROM memory.
@@ -405,7 +408,7 @@ public:
       @param  outBytes     Values that were reveived. Read length is determined
                            by the preset length of the vector.
    */
-  void cmd_ReadMemory( const uint8_t inStartAddr, byteVec_t &outBytes) const;
+  virtual void cmd_ReadMemory( const uint8_t inStartAddr, byteVec_t &outBytes) const;
 
   
   //cmd_ExtendedReadMemory();  // Not implemented yet
@@ -449,7 +452,7 @@ public:
 
       @throw devDS2406_error  Thrown when illegal address is selected.
    */  
-  void cmd_ReadStatus( const uint8_t inStartAddr, byteVec_t &outBytes) const;
+  virtual void cmd_ReadStatus( const uint8_t inStartAddr, byteVec_t &outBytes) const;
 
    
   /** Write to status memory.
@@ -475,7 +478,7 @@ public:
 
       @throw devDS2406_error  Thrown when illegal address is selected.
    */
-  void cmd_WriteStatus( const uint8_t inStartAddr, const byteVec_t &inWriteBytes) const;
+  virtual void cmd_WriteStatus( const uint8_t inStartAddr, const byteVec_t &inWriteBytes) const;
     
 
     
@@ -528,8 +531,8 @@ protected:
 
       @throw devDS2406_error  Thrown when illegal address is selected.
    */
-  void readMemUniversal( const uint16_t inStartAddr, byteVec_t &outBytes, 
-                         const uint16_t inMaxLen, const owCommand_t inCommand) const;
+  virtual void readMemUniversal( const uint16_t inStartAddr, byteVec_t &outBytes, 
+                                 const uint16_t inMaxLen, const owCommand_t inCommand) const;
   
   
   
