@@ -2,8 +2,8 @@
                           LOW_platformMiscFactory.cpp  -  description
                              -------------------
     begin                : Thu Aug 1 2002
-    copyright            : (C) 2002 by Harald Roelle, Helmut Reiser
-    email                : roelle@informatik.uni-muenchen.de, reiser@informatik.uni-muenchen.de
+    copyright            : (C) 2002 by Harald Roelle
+    email                : roelle@informatik.uni-muenchen.de
  ***************************************************************************/
 
 /***************************************************************************
@@ -21,6 +21,21 @@
 
 
 
+//=====================================================================================
+//
+// constructors
+//
+
+LOW_platformMiscFactory::LOW_platformMiscFactory()
+{
+}
+
+
+LOW_platformMiscFactory::~LOW_platformMiscFactory()
+{
+}
+
+
 //=====================================================================================
 //
 // static factory
