@@ -2,8 +2,8 @@
                           test_ds2406.h  -  description
                              -------------------
     begin                : Wed Jan 8 2003
-    copyright            : (C) 2003 by Harald Roelle, Helmut Reiser
-    email                : roelle@informatik.uni-muenchen.de, reiser@informatik.uni-muenchen.de
+    copyright            : (C) 2003 by Harald Roelle
+    email                : roelle@informatik.uni-muenchen.de
  ***************************************************************************/
 
 /***************************************************************************
